export let htmlIntro = `<p>Welcome to Peek.</p>

            <p>Peek Field Swithing allows field engineers working on the
                electricity network to electronically</p>
            <ul>
                <li>View switching instructions</li>
                <li>Receive instructions from the control room</li>
                <li>And field confirm field switching operations</li>
            </ul>`;