@import '~@synerty/peek-theme/scss/_variables.web';

$general_margin_1: 1 !default;
$general_font_size_5: 1 !default;
$general_margin_3: 1 !default;
$general_padding_3: 1 !default;

.plugin-user {
  /*
      Contains the user component classes
  */
  margin: $general_margin_1;

  .user-message {
    /*
        Contains the message attributes
    */
    font-size: $general_font_size_5;
    margin-top: $general_margin_3;

  }
}

.peek-nav-section {
  position: relative;
}